from src.abstract_ai import *
input(isinstance({'api_response': 'Absolutely! Understanding the complexities requires in-depth exploration beyond this summary.', 'generate_title': 'Understanding_Complexities_In-Depth_Exploration'},dict))
result = make_general_query("""Absolutely! Understanding the complexitiesin-depth exploration beyond this summary.""")
input(result)
"""/home/computron/Documents/pythonTools/modules/abstract_ai/src/abstract_ai/gpt_classes/nogui_selection/query_all.py:from abstract_ai import *
/home/computron/Documents/pythonTools/modules/abstract_ai/src/abstract_ai/gpt_classes/nogui_selection/prompt_manager.py:from abstract_ai.gpt_classes.model_selection.ModelBuilder import ModelManager
/home/computron/Documents/pythonTools/modules/abstract_ai/src/abstract_ai/gpt_classes/nogui_selection/db_query.py:from abstract_ai import async_get_xlsx_file_path_from_response_vars
/home/computron/Documents/pythonTools/modules/abstract_ai/src/abstract_ai/ai_call_package/apis/make_api_calls.py:from abstract_ai import *
/home/computron/Documents/pythonTools/modules/abstract_ai/src/abstract_ai/specializations/database_query.py:from abstract_ai.gpt_classes.nogui_selection.general_query import make_general_query
/home/computron/Documents/pythonTools/modules/abstract_ai/src/abstract_ai/specializations/images/dalee_image_create.py:from abstract_ai.gpt_classes import ApiManager
"""



tests='''#ResponseManagement
    def update_response_mgr(self)->None:
        self.response_mgr = ResponseManager(prompt_mgr=self.prompt_mgr,api_mgr=self.api_mgr)
        
    def get_new_api_call_name(self)->None:
        """
        A helper method that generates and appends a new unique API call name to the api_call_list.
 	"""
        call_name = create_new_name(name='api_call',names_list=self.api_call_list)
        if call_name not in self.api_call_list:
            self.api_call_list.append(call_name)

    def start_asyncio_loop(self):
        asyncio.set_event_loop(self.loop)
        self.loop.run_forever()    
    def query_event_check(self) -> bool:
        if self.event == "-SUBMIT_QUERY-":
            if not self.submission_in_progress:
                self.submission_in_progress = True
                self.get_new_api_call_name()
                # Submit the coroutine to the event loop
                future = asyncio.run_coroutine_threadsafe(self.submit_query(), self.loop)
                # Optionally, handle the result or exceptions
                future.add_done_callback(self.handle_future_result)
            else:
                self.logger.info("Submission already in progress.")
            self.event = None
        else:
            return False
        return True

    def handle_future_result(self, future):
        try:
            result = future.result()
            # Process the result if needed
        except Exception as e:
            self.logger.error(f"Error in async task: {e}")
    def submit_event(self):
        self.get_new_api_call_name()
        self.start_query = False
        self.updated_progress = False
        self.response_mgr.re_initialize_query()
        asyncio.run(self.submit_query())
        self.submission_in_progress = False  # Reset the flag when done
    
    def get_dots(self)->None:
        """
        This is a helper method used in the update_progress_chunks method. It manages the progress tracking visual
        indication in the UI. It updates the dots string depending on the current state of the dots. If the dots string
        is full of dots, it resets it to empty.
        """
        count = 0
        stop = False
        dots = ''
        for each in self.dots: 
            if each == ' ' and stop == False:
                dots+='.'
                stop = True
            else:
                dots+=each
        self.dots = dots
        if stop == False:
            self.dots = '   '
        get_sleep(1)
        status='Testing'
        if self.test_bool == False:
            status = "Updating Content" if not self.updated_progress else "Sending"
        self.window_mgr.update_value(key='-PROGRESS_TEXT-', value=f'{status}{self.dots}')
        
    def update_progress_chunks(self,done:bool=False)->None:
        """
        updates the progress bar and text status in the GUI, depending on the amount of chunks processed. If the
        process is completed, it updates the UI with a sent status and enables the '-SUBMIT_QUERY-' button again.
        uses the helper get_dots method for dynamic visual indication of the processing.
	"""
        self.total_dists=0
        for dist in self.chunk_token_distributions:
            self.total_dists +=len(dist)
        i_query = int(self.response_mgr.i_query)
        if done == True:
            self.window['-PROGRESS-'].update_bar(100, 100)
            self.window_mgr.update_value(key='-QUERY_COUNT-', value=f"a total of {self.total_dists} chunks have been sent")
            self.window_mgr.update_value(key='-PROGRESS_TEXT-', value='SENT')
            self.updated_progress = True
        else:
            self.get_dots()
            self.window['-PROGRESS-'].update_bar(i_query, self.total_dists)
            if i_query == 0 and self.total_dists!= 0 :
                i_query = 1
            self.window_mgr.update_value(key='-QUERY_COUNT-', value=f"chunk {i_query} of {self.total_dists} being sent")
            
    # checks if the Response Manager has completed the query process.         
    def check_response_mgr_status(self)->bool:
        """
        checks if the Response Manager has completed the query process. It returns a Boolean indicating the status.
	"""
        if not self.test_bool:
            return self.response_mgr.query_done
        return self.start_query
    
    # Manages the process of submitting queries to the GPT AI
    async def submit_query(self) -> None:
        try:
            # Update the GUI components from the main thread using a thread-safe method
            self.window.write_event_value('-DISABLE_SUBMIT_BUTTON-', True)

            self.dots = '...'
            self.start_query = False
            if self.test_bool:
                self.latest_output = [safe_read_from_json(self.test_file_path)]
            else:
                self.latest_output = await self.get_query() or []
            self.output_list.append(self.latest_output)
            self.logger.error(f"recent output_list: {self.output_list}")
            self.update_progress_chunks(done=True)
            self.update_last_response_file()
            self.update_text_with_responses()
            # Re-enable the submit button
            self.window.write_event_value('-ENABLE_SUBMIT_BUTTON-', True)
            if not self.window_mgr.get_values()['-REUSE_CHUNK_DATA-']:
                self.update_prompt_data('')
            self.response = False
        finally:
            self.submission_in_progress = False
        
        
    async def get_query(self) -> None:
        self.response = await self.response_mgr.initial_query()
        return self.response
    def response_event_check(self)->bool:
        """
        Checks if a response event has been triggered.

        A response event is triggered when a user interacts with the response management interface.

        Returns:
            bool: True if a response event has been triggered, otherwise False
        """
        if self.event == "-COLLATE_RESPONSES_BOOL-":
            if self.window_mgr.get_values()["-COLLATE_RESPONSES_BOOL-"]:
                files_list = self.window['-FILES_LIST_RESPONSES-'].Values
                if len(files_list)>0:
                    collator = FileCollator(files_list)
                    collated_responses=collator.get_gollated_responses(files_list,key_value = 'response')
                    self.window_mgr.update_value('-FILE_TEXT_RESPONSES-',collated_responses)
                    
        elif self.event in ['-RESPONSE_TEXT_BACK-','-RESPONSE_TEXT_FORWARD-']:
            self.determine_output_display(self.event)
            
        elif self.event == 'Copy Response':
            active_tab_key = self.window_mgr.get_values()['-TABS-']  # get active tab key
            # Construct the key for multiline text box
            multiline_key = active_tab_key.replace('TAB','TEXT')
            if multiline_key in self.window_mgr.get_values():
                text_to_copy = self.window_mgr.get_values()['-FILE_TEXT-']
                pyperclip.copy(text_to_copy)
        else:
            return False
        return True
    
    def get_output_response(self,model:str="None",title:str="None",request:str="None",response:str="None",output_files:str="None")->str:
        """
        A helper method that formats the output string of a response in a clear and readable manner.
        It includes information about the model, title, user request, and AI response.
 	"""
        model = f"#MODEL#{self.get_new_line(1)}{self.last_model}{self.get_new_line(2)}"
        title = f"#TITLE#{self.get_new_line(1)}{str(title)}{self.get_new_line(2)}"
        request = f"#USER REQUEST#{self.get_new_line(1)}{str(request)}{self.get_new_line(2)}"
        response = f"#RESPONSE#{self.get_new_line(2)}{str(response)}{self.get_new_line(2)}"
        output_files = f"#OUTPUT FILES#{self.get_new_line(1)}{str(output_files)}{self.get_new_line(2)}"
        return f"{model}{title}{request}{response}{output_files}"
    
    def update_text_with_responses(self)->None:
        """
        gets the latest response from the AI, formats it using the get_output_response helper method,
        and updates the GUI's '-RESPONSE-' key with the formatted string.
 	"""
        output_keys = []
        self.last_prompt= get_any_value(safe_json_loads(self.current_output),'prompt')
        self.last_response = get_response(self.current_output)
        self.last_file_path= get_any_data(self.current_output,'file_path')
        self.last_title= get_title(self.current_output)
        self.unlist_self(["last_prompt","last_title","last_file_path","last_prompt"])
        self.last_content = get_any_data(self.last_response,'content')
        self.last_api_response = get_api_response(self.last_response)
        self.last_model =get_any_value(safe_json_loads(self.last_response),'model')
        self.unlist_self(["last_model","last_api_response","last_content"])
        self.window_mgr.update_value(key=text_to_key('title input'),value=self.last_title)
        self.database_output_file = generate_query_from_recent_response(file_path=self.last_file_path)
        response_content = self.last_content
        if self.last_api_response:
            response_content=self.last_api_response
            self.delegate_instruction_text()
        self.window_mgr.update_value('-RESPONSE-',self.get_output_response(model=self.last_model,
                                                                           title=self.last_title,
                                                                           request=self.request_data_list[self.get_current_request_data()],
                                                                           response=response_content,output_files=self.database_output_file))
        
    def get_current_request_data(self)->int:
        num=0
        for i,data in enumerate(self.prompt_data_list):
            num+=len(data)
            if self.response_text_number_actual<=num:
                if i>len(self.request_data_list):
                    return len(request_data_list)-1
                return i
        return len(self.request_data_list)-1
    def unlist_self(self,keys:list):
        """
        A helper method that takes a list of keys as a parameter, iterates over each key, and checks if
        the value of the corresponding key in the class is a non-empty list. If it is, it updates the class's attribute with the last element in the list.
 	"""
        for key in keys:
            value = getattr(self,key)
            if isinstance(value,list) and value:
                value = value[-1]
            setattr(self,key,value)
    def initialize_output_display(self)->None:
        """
        Initializes the display of the latest output in the GUI.
        """
        if len(self.latest_output)>0:
            self.current_output=self.latest_output[-1]
        else:
            self.current_output=[]
        self.response_text_number_actual=0
        self.response_text_number_display=1
        self.window_mgr.update_value(key='-RESPONSE_TEXT_NUMBER-',value=self.response_text_number_display)
        self.update_output(output_iteration=0)
        
    def get_output_display_numbers(self)->None:
        """
        Retrieves the actual and display numbers of the current output for navigation purposes.
        """
        self.response_text_number_display = int(self.window_mgr.get_from_value('-RESPONSE_TEXT_NUMBER-'))
        self.response_text_number_actual = self.response_text_number_display-1
        
    def determine_output_display(self,event):
        """
        Determines the output to display based on the navigation event.

        Args:
            event: The navigation event that occurred
        """
        self.get_output_display_numbers()
        if self.event == '-RESPONSE_TEXT_BACK-':
            if self.response_text_number_actual >0:
                self.adjust_output_display(-1)
        elif self.event == '-RESPONSE_TEXT_FORWARD-':
            if self.response_text_number_display < len(self.latest_output):
                self.adjust_output_display(1)
                
    def update_output(self,output_iteration:int):
        """
        Updates the displayed output based on the given iteration.

        Args:
            output_iteration (int): The index of the output to display.
        """
        if output_iteration < len(self.latest_output) and output_iteration >=0:
            self.current_output = self.latest_output[output_iteration]
            self.update_text_with_responses()
            
    def adjust_output_display(self,num:int):
        self.response_text_number_actual+=num
        self.response_text_number_display+=num
        self.update_output(output_iteration=self.response_text_number_actual)
        self.window_mgr.update_value(key='-RESPONSE_TEXT_NUMBER-',value=self.response_text_number_display)
        
    def listbox_event_check(self)->bool:
        """
        Checks if a listbox event has been triggered and updates the GUI accordingly.

        Returns:
            bool: True if a listbox event has been triggered, otherwise False
        """
        event_triggered = False
        values = self.window_mgr.get_values()
        file_path = None

        if self.script_event_js['found'] == '-FILES_LIST-':
            selected_files = values[self.event]
            if selected_files:
                file_path = selected_files[0]
                event_triggered = True
        elif self.script_event_js['found'] == '-BROWSER_LIST-':
            directory = values[self.script_event_js['-DIR-']]
            selected_files = values[self.script_event_js['-BROWSER_LIST-']]
            if selected_files:
                file_path = os.path.join(directory, selected_files[0])
                event_triggered = True
        if self.window_mgr.get_from_value(text_to_key('database query bool', section='database')):
                contents = get_instruction_from_tableName(self.window_mgr.get_from_value(text_to_key('table configuration', section='database')))
                self.window_mgr.update_value(key=text_to_key('-FILE_TEXT-', section='database'), value=contents)
                self.window_mgr.update_value(key=text_to_key('-CHUNK_TITLE-', section='database'), value=self.chunk_title)
        if file_path:
            if os.path.isfile(file_path):
                try:
                    contents = self.read_file_contents(file_path)  # Assuming read_file_contents is defined to handle different file types
                    contents = safe_json_loads(contents)
                    
                    if isinstance(contents,dict) and self.window_mgr.get_from_value(text_to_key('format json to string', section='responses')):
                        contents = format_json_key_values(contents)
                    self.window_mgr.update_value(key=self.script_event_js['-FILE_TEXT-'], value=contents)
                    self.chunk_title = os.path.basename(file_path)
                    self.window_mgr.update_value(key=text_to_key('-CHUNK_TITLE-', section='files'), value=self.chunk_title)
                except Exception as e:
                    print(f"Cannot read from path {file_path}. Error: {e}")
            else:
                file_path = self.window_mgr.get_values()[self.script_event_js['-DIR-']]
                files_list = self.window_mgr.get_values()[self.script_event_js['-BROWSER_LIST-']]
                if file_path:
                    file_path = os.path.join(file_path,files_list[0])
                    if not os.path.isfile(file_path):
                        self.browser_mgr.handle_event(self.event,self.window_mgr.get_values(),self.window)
        return event_triggered

    get_instruction_from_tableName()
    def update_last_response_file(self)->None:
        """
        Updates the path to the last file that recorded a response.

        This function also updates the GUI with the contents of the new file.
        """
        if self.test_bool:
            self.last_response_file=self.test_file_path
        else:
            self.last_response_file = self.response_mgr.save_manager.file_path
        self.last_directory=os.path.dirname(self.last_response_file)
        self.window_mgr.update_value(key='-DIR_RESPONSES-',value=self.last_directory)
        self.update_response_list_box()
        self.window["-DIRECTORY_BROWSER_RESPONSES-"].InitialFolder=self.last_directory
        self.window["-FILE_BROWSER_RESPONSES-"].InitialFolder=self.last_directory
        self.initialize_output_display()
        
    def update_response_list_box(self)->None:
        """
        Update the List Box component that displays saved responses.

        This function is often called after a new API response has been received and saved.
        """
        if self.latest_output:
            for data in self.latest_output:
                file_path = get_any_value(data,'file_path')
                if file_path in self.new_response_path_list:
                    self.new_response_path_list.remove(file_path)
                self.new_response_path_list.append(file_path)
            files_list = self.window['-FILES_LIST_RESPONSES-'].Values
            if file_path in self.new_response_path_list:
                if file_path in files_list:
                    files_list.remove(file_path)
            files_list=self.new_response_path_list+files_list
            self.window_mgr.update_value('-FILES_LIST_RESPONSES-',args={"values":files_list})

    def aggregate_conversations(self,directory:str=None)->list:
        """
        Aggregates conversations from JSON files in the specified directory.

        Args:
            directory (str): The directory containing the JSON files. If none, it will take the current working directory.

        Returns:
            list: A list of aggregated conversations.
        """
        if directory == None:
            directory = os.getcwd()
        json_files, lsAll = [], []
        for root, _, files in os.walk(directory):
            for file in files:
                if file.endswith(".json"):
                    file = os.path.join(root, file)
                    json_files.append((get_file_create_time(file), file))
        sorted_json_files = sorted(json_files, key=lambda x: x[0])
        aggregated_conversations = []
        chat_log_file = open("chat_log.txt", "w")
        model = 'gpt'
        for file in sorted_json_files:
            file = file[1]
            data = safe_read_from_json(file)
            if data:
                relevant_values={}
                relevant_keys = ['id', 'object', 'created', 'model']
                for key in relevant_keys:
                    value = get_any_value(data,key)
                    if value:
                        relevant_values[key]=value
                if len(relevant_values.keys())==len(relevant_keys):
                    if file not in aggregated_conversations:
                        aggregated_conversations.append(file)
                    dir_name=os.path.dirname(file)
                    if dir_name not in self.response_directories_list:
                        self.response_directories_list.append(dir_name)
           
        #self.next_read_mgr.add_to_queue(self.window_mgr.update_value,'-FILES_LIST_RESPONSES-',args={"values":aggregated_conversations})
        return aggregated_conversations'''
manager_name = 'GptManager'
for test in tests.split('def ')[1:]:
        function_name = test.split('(')[0]
        print(f"self.{function_name}={function_name}")
        



{
  "object": "list",
  "data": [
    {
      "id": "text-search-babbage-doc-001",
      "object": "model",
      "created": 1651172509,
      "owned_by": "openai-dev",
      "permission": [
        {
          "id": "modelperm-s9n5HnzbtVn7kNc5TIZWiCFS",
          "object": "model_permission",
          "created": 1695933794,
          "allow_create_engine": false,
          "allow_sampling": true,
          "allow_logprobs": true,
          "allow_search_indices": true,
          "allow_view": true,
          "allow_fine_tuning": false,
          "organization": "*",
          "group": null,
          "is_blocking": false
        }
      ],
      "root": "text-search-babbage-doc-001",
      "parent": null
    },
    {
      "id": "curie-search-query",
      "object": "model",
      "created": 1651172509,
      "owned_by": "openai-dev",
      "permission": [
        {
          "id": "modelperm-8aqdyZaKtD3MD831mGbqh1MD",
          "object": "model_permission",
          "created": 1695149182,
          "allow_create_engine": false,
          "allow_sampling": true,
          "allow_logprobs": true,
          "allow_search_indices": true,
          "allow_view": true,
          "allow_fine_tuning": false,
          "organization": "*",
          "group": null,
          "is_blocking": false
        }
      ],
      "root": "curie-search-query",
      "parent": null
    },
    {
      "id": "text-search-babbage-query-001",
      "object": "model",
      "created": 1651172509,
      "owned_by": "openai-dev",
      "permission": [
        {
          "id": "modelperm-hXsRH2IK0hXmWxmLRiNTp70t",
          "object": "model_permission",
          "created": 1695933813,
          "allow_create_engine": false,
          "allow_sampling": true,
          "allow_logprobs": true,
          "allow_search_indices": true,
          "allow_view": true,
          "allow_fine_tuning": false,
          "organization": "*",
          "group": null,
          "is_blocking": false
        }
      ],
      "root": "text-search-babbage-query-001",
      "parent": null
    },
    {
      "id": "babbage",
      "object": "model",
      "created": 1649358449,
      "owned_by": "openai",
      "permission": [
        {
          "id": "modelperm-h574xGeqWyBeFDDKaoVTC4CO",
          "object": "model_permission",
          "created": 1692394129,
          "allow_create_engine": false,
          "allow_sampling": true,
          "allow_logprobs": true,
          "allow_search_indices": false,
          "allow_view": true,
          "allow_fine_tuning": false,
          "organization": "*",
          "group": null,
          "is_blocking": false
        }
      ],
      "root": "babbage",
      "parent": null
    },
    {
      "id": "babbage-search-query",
      "object": "model",
      "created": 1651172509,
      "owned_by": "openai-dev",
      "permission": [
        {
          "id": "modelperm-1zMLcaRlTvYAdpmvvixnTWlF",
          "object": "model_permission",
          "created": 1695933835,
          "allow_create_engine": false,
          "allow_sampling": true,
          "allow_logprobs": true,
          "allow_search_indices": true,
          "allow_view": true,
          "allow_fine_tuning": false,
          "organization": "*",
          "group": null,
          "is_blocking": false
        }
      ],
      "root": "babbage-search-query",
      "parent": null
    },
    {
      "id": "text-babbage-001",
      "object": "model",
      "created": 1649364043,
      "owned_by": "openai",
      "permission": [
        {
          "id": "modelperm-YABzYWjC1kS6M2BnI6Fr9vuS",
          "object": "model_permission",
          "created": 1690913878,
          "allow_create_engine": false,
          "allow_sampling": true,
          "allow_logprobs": true,
          "allow_search_indices": false,
          "allow_view": true,
          "allow_fine_tuning": false,
          "organization": "*",
          "group": null,
          "is_blocking": false
        }
      ],
      "root": "text-babbage-001",
      "parent": null
    },
    {
      "id": "text-similarity-davinci-001",
      "object": "model",
      "created": 1651172505,
      "owned_by": "openai-dev",
      "permission": [
        {
          "id": "modelperm-C6TT4mQR3bJQEzEuiZlhKM5u",
          "object": "model_permission",
          "created": 1695143463,
          "allow_create_engine": false,
          "allow_sampling": true,
          "allow_logprobs": true,
          "allow_search_indices": true,
          "allow_view": true,
          "allow_fine_tuning": false,
          "organization": "*",
          "group": null,
          "is_blocking": false
        }
      ],
      "root": "text-similarity-davinci-001",
      "parent": null
    },
    {
      "id": "davinci",
      "object": "model",
      "created": 1649359874,
      "owned_by": "openai",
      "permission": [
        {
          "id": "modelperm-RcfCH2MkO5NP9C7wx6kdYnIT",
          "object": "model_permission",
          "created": 1694819047,
          "allow_create_engine": false,
          "allow_sampling": true,
          "allow_logprobs": true,
          "allow_search_indices": false,
          "allow_view": true,
          "allow_fine_tuning": false,
          "organization": "*",
          "group": null,
          "is_blocking": false
        }
      ],
      "root": "davinci",
      "parent": null
    },
    {
      "id": "davinci-similarity",
      "object": "model",
      "created": 1651172509,
      "owned_by": "openai-dev",
      "permission": [
        {
          "id": "modelperm-OtRWxI0nRtN9q8mHI3OOk0GT",
          "object": "model_permission",
          "created": 1695143471,
          "allow_create_engine": false,
          "allow_sampling": true,
          "allow_logprobs": true,
          "allow_search_indices": true,
          "allow_view": true,
          "allow_fine_tuning": false,
          "organization": "*",
          "group": null,
          "is_blocking": false
        }
      ],
      "root": "davinci-similarity",
      "parent": null
    },
    {
      "id": "code-davinci-edit-001",
      "object": "model",
      "created": 1649880484,
      "owned_by": "openai",
      "permission": [
        {
          "id": "modelperm-T8Ie7SvlPyvtsDvPlfC8DftZ",
          "object": "model_permission",
          "created": 1690915089,
          "allow_create_engine": false,
          "allow_sampling": true,
          "allow_logprobs": true,
          "allow_search_indices": false,
          "allow_view": true,
          "allow_fine_tuning": false,
          "organization": "*",
          "group": null,
          "is_blocking": false
        }
      ],
      "root": "code-davinci-edit-001",
      "parent": null
    },
    {
      "id": "curie-similarity",
      "object": "model",
      "created": 1651172510,
      "owned_by": "openai-dev",
      "permission": [
        {
          "id": "modelperm-9PfIlYDKOt24EV6fnCiunZGA",
          "object": "model_permission",
          "created": 1695149271,
          "allow_create_engine": false,
          "allow_sampling": true,
          "allow_logprobs": true,
          "allow_search_indices": true,
          "allow_view": true,
          "allow_fine_tuning": false,
          "organization": "*",
          "group": null,
          "is_blocking": false
        }
      ],
      "root": "curie-similarity",
      "parent": null
    },
    {
      "id": "babbage-search-document",
      "object": "model",
      "created": 1651172510,
      "owned_by": "openai-dev",
      "permission": [
        {
          "id": "modelperm-qJ0Iu5XcwrdsmSOn9ewphBNF",
          "object": "model_permission",
          "created": 1695933877,
          "allow_create_engine": false,
          "allow_sampling": true,
          "allow_logprobs": true,
          "allow_search_indices": true,
          "allow_view": true,
          "allow_fine_tuning": false,
          "organization": "*",
          "group": null,
          "is_blocking": false
        }
      ],
      "root": "babbage-search-document",
      "parent": null
    },
    {
      "id": "curie-instruct-beta",
      "object": "model",
      "created": 1649364042,
      "owned_by": "openai",
      "permission": [
        {
          "id": "modelperm-rTxpdy2DwwUp38frYQFsj5OC",
          "object": "model_permission",
          "created": 1694819118,
          "allow_create_engine": false,
          "allow_sampling": true,
          "allow_logprobs": true,
          "allow_search_indices": false,
          "allow_view": true,
          "allow_fine_tuning": false,
          "organization": "*",
          "group": null,
          "is_blocking": false
        }
      ],
      "root": "curie-instruct-beta",
      "parent": null
    },
    {
      "id": "text-search-ada-doc-001",
      "object": "model",
      "created": 1651172507,
      "owned_by": "openai-dev",
      "permission": [
        {
          "id": "modelperm-tdL3cX2rMgQuyfQwqNGcQOp3",
          "object": "model_permission",
          "created": 1695166106,
          "allow_create_engine": false,
          "allow_sampling": true,
          "allow_logprobs": true,
          "allow_search_indices": true,
          "allow_view": true,
          "allow_fine_tuning": false,
          "organization": "*",
          "group": null,
          "is_blocking": false
        }
      ],
      "root": "text-search-ada-doc-001",
      "parent": null
    },
    {
      "id": "davinci-instruct-beta",
      "object": "model",
      "created": 1649364042,
      "owned_by": "openai",
      "permission": [
        {
          "id": "modelperm-SKEW42qz05X43sqLXFwhbAox",
          "object": "model_permission",
          "created": 1695166658,
          "allow_create_engine": false,
          "allow_sampling": true,
          "allow_logprobs": true,
          "allow_search_indices": false,
          "allow_view": true,
          "allow_fine_tuning": false,
          "organization": "*",
          "group": null,
          "is_blocking": false
        }
      ],
      "root": "davinci-instruct-beta",
      "parent": null
    },
    {
      "id": "text-similarity-babbage-001",
      "object": "model",
      "created": 1651172505,
      "owned_by": "openai-dev",
      "permission": [
        {
          "id": "modelperm-yNsfFUtfAoDhJHRofgD2Kjgd",
          "object": "model_permission",
          "created": 1695933939,
          "allow_create_engine": false,
          "allow_sampling": true,
          "allow_logprobs": true,
          "allow_search_indices": true,
          "allow_view": true,
          "allow_fine_tuning": false,
          "organization": "*",
          "group": null,
          "is_blocking": false
        }
      ],
      "root": "text-similarity-babbage-001",
      "parent": null
    },
    {
      "id": "text-search-davinci-doc-001",
      "object": "model",
      "created": 1651172505,
      "owned_by": "openai-dev",
      "permission": [
        {
          "id": "modelperm-I0hY9ySAeJrE3qBF47roClh9",
          "object": "model_permission",
          "created": 1695143675,
          "allow_create_engine": false,
          "allow_sampling": true,
          "allow_logprobs": true,
          "allow_search_indices": true,
          "allow_view": true,
          "allow_fine_tuning": false,
          "organization": "*",
          "group": null,
          "is_blocking": false
        }
      ],
      "root": "text-search-davinci-doc-001",
      "parent": null
    },
    {
      "id": "babbage-similarity",
      "object": "model",
      "created": 1651172505,
      "owned_by": "openai-dev",
      "permission": [
        {
          "id": "modelperm-8Inp4vh9P5Mh0llHmSX8Va3b",
          "object": "model_permission",
          "created": 1695933959,
          "allow_create_engine": false,
          "allow_sampling": true,
          "allow_logprobs": true,
          "allow_search_indices": true,
          "allow_view": true,
          "allow_fine_tuning": false,
          "organization": "*",
          "group": null,
          "is_blocking": false
        }
      ],
      "root": "babbage-similarity",
      "parent": null
    },
    {
      "id": "gpt-3.5-turbo-0613",
      "object": "model",
      "created": 1686587434,
      "owned_by": "openai",
      "permission": [
        {
          "id": "modelperm-i2x0QNY071EgBaeuNj5ftNhX",
          "object": "model_permission",
          "created": 1697243388,
          "allow_create_engine": false,
          "allow_sampling": true,
          "allow_logprobs": true,
          "allow_search_indices": false,
          "allow_view": true,
          "allow_fine_tuning": false,
          "organization": "*",
          "group": null,
          "is_blocking": false
        }
      ],
      "root": "gpt-3.5-turbo-0613",
      "parent": null
    },
    {
      "id": "davinci-search-query",
      "object": "model",
      "created": 1651172505,
      "owned_by": "openai-dev",
      "permission": [
        {
          "id": "modelperm-ZNL7KVorrZk5pvj7v3cG3lUz",
          "object": "model_permission",
          "created": 1695143683,
          "allow_create_engine": false,
          "allow_sampling": true,
          "allow_logprobs": true,
          "allow_search_indices": true,
          "allow_view": true,
          "allow_fine_tuning": false,
          "organization": "*",
          "group": null,
          "is_blocking": false
        }
      ],
      "root": "davinci-search-query",
      "parent": null
    },
    {
      "id": "text-similarity-curie-001",
      "object": "model",
      "created": 1651172507,
      "owned_by": "openai-dev",
      "permission": [
        {
          "id": "modelperm-J2U414W9ZcjxYqf13tZdvwI5",
          "object": "model_permission",
          "created": 1695149279,
          "allow_create_engine": false,
          "allow_sampling": true,
          "allow_logprobs": true,
          "allow_search_indices": true,
          "allow_view": true,
          "allow_fine_tuning": false,
          "organization": "*",
          "group": null,
          "is_blocking": false
        }
      ],
      "root": "text-similarity-curie-001",
      "parent": null
    },
    {
      "id": "text-davinci-001",
      "object": "model",
      "created": 1649364042,
      "owned_by": "openai",
      "permission": [
        {
          "id": "modelperm-bwpDudS41UaZskP1jKfAHKMG",
          "object": "model_permission",
          "created": 1694819202,
          "allow_create_engine": false,
          "allow_sampling": true,
          "allow_logprobs": true,
          "allow_search_indices": false,
          "allow_view": true,
          "allow_fine_tuning": false,
          "organization": "*",
          "group": null,
          "is_blocking": false
        }
      ],
      "root": "text-davinci-001",
      "parent": null
    },
    {
      "id": "text-search-davinci-query-001",
      "object": "model",
      "created": 1651172505,
      "owned_by": "openai-dev",
      "permission": [
        {
          "id": "modelperm-lEI8RY9UBNhXYJRKub8U4Sc1",
          "object": "model_permission",
          "created": 1695143690,
          "allow_create_engine": false,
          "allow_sampling": true,
          "allow_logprobs": true,
          "allow_search_indices": true,
          "allow_view": true,
          "allow_fine_tuning": false,
          "organization": "*",
          "group": null,
          "is_blocking": false
        }
      ],
      "root": "text-search-davinci-query-001",
      "parent": null
    },
    {
      "id": "ada-search-document",
      "object": "model",
      "created": 1651172507,
      "owned_by": "openai-dev",
      "permission": [
        {
          "id": "modelperm-CSgBXBOV02tB05EbOK36apFu",
          "object": "model_permission",
          "created": 1695166132,
          "allow_create_engine": false,
          "allow_sampling": true,
          "allow_logprobs": true,
          "allow_search_indices": true,
          "allow_view": true,
          "allow_fine_tuning": false,
          "organization": "*",
          "group": null,
          "is_blocking": false
        }
      ],
      "root": "ada-search-document",
      "parent": null
    },
    {
      "id": "ada-code-search-code",
      "object": "model",
      "created": 1651172505,
      "owned_by": "openai-dev",
      "permission": [
        {
          "id": "modelperm-NFpO3yF5OZ9bfwls2MSUn5cR",
          "object": "model_permission",
          "created": 1695166330,
          "allow_create_engine": false,
          "allow_sampling": true,
          "allow_logprobs": true,
          "allow_search_indices": true,
          "allow_view": true,
          "allow_fine_tuning": false,
          "organization": "*",
          "group": null,
          "is_blocking": false
        }
      ],
      "root": "ada-code-search-code",
      "parent": null
    },
    {
      "id": "babbage-002",
      "object": "model",
      "created": 1692634615,
      "owned_by": "system",
      "permission": [
        {
          "id": "modelperm-tC8uL3ohDEVSzUvcgZCv33gV",
          "object": "model_permission",
          "created": 1692720928,
          "allow_create_engine": false,
          "allow_sampling": true,
          "allow_logprobs": true,
          "allow_search_indices": false,
          "allow_view": true,
          "allow_fine_tuning": false,
          "organization": "*",
          "group": null,
          "is_blocking": false
        }
      ],
      "root": "babbage-002",
      "parent": null
    },
    {
      "id": "davinci-002",
      "object": "model",
      "created": 1692634301,
      "owned_by": "system",
      "permission": [
        {
          "id": "modelperm-9a6yBfznsbObxHVWBWRJctsx",
          "object": "model_permission",
          "created": 1694818100,
          "allow_create_engine": false,
          "allow_sampling": true,
          "allow_logprobs": true,
          "allow_search_indices": false,
          "allow_view": true,
          "allow_fine_tuning": false,
          "organization": "*",
          "group": null,
          "is_blocking": false
        }
      ],
      "root": "davinci-002",
      "parent": null
    },
    {
      "id": "davinci-search-document",
      "object": "model",
      "created": 1651172509,
      "owned_by": "openai-dev",
      "permission": [
        {
          "id": "modelperm-Bbb0MBYeWROhvx9NkaDbZg3R",
          "object": "model_permission",
          "created": 1695143700,
          "allow_create_engine": false,
          "allow_sampling": true,
          "allow_logprobs": true,
          "allow_search_indices": true,
          "allow_view": true,
          "allow_fine_tuning": false,
          "organization": "*",
          "group": null,
          "is_blocking": false
        }
      ],
      "root": "davinci-search-document",
      "parent": null
    },
    {
      "id": "curie-search-document",
      "object": "model",
      "created": 1651172508,
      "owned_by": "openai-dev",
      "permission": [
        {
          "id": "modelperm-xkoHDxWoBlLQj4HM4iVwWj9C",
          "object": "model_permission",
          "created": 1695149161,
          "allow_create_engine": false,
          "allow_sampling": true,
          "allow_logprobs": true,
          "allow_search_indices": true,
          "allow_view": true,
          "allow_fine_tuning": false,
          "organization": "*",
          "group": null,
          "is_blocking": false
        }
      ],
      "root": "curie-search-document",
      "parent": null
    },
    {
      "id": "babbage-code-search-code",
      "object": "model",
      "created": 1651172509,
      "owned_by": "openai-dev",
      "permission": [
        {
          "id": "modelperm-5Dv3knU1yoRtpY5TFzSHySiM",
          "object": "model_permission",
          "created": 1695934020,
          "allow_create_engine": false,
          "allow_sampling": true,
          "allow_logprobs": true,
          "allow_search_indices": true,
          "allow_view": true,
          "allow_fine_tuning": false,
          "organization": "*",
          "group": null,
          "is_blocking": false
        }
      ],
      "root": "babbage-code-search-code",
      "parent": null
    },
    {
      "id": "text-search-ada-query-001",
      "object": "model",
      "created": 1651172505,
      "owned_by": "openai-dev",
      "permission": [
        {
          "id": "modelperm-2XH9mphBxBD5uKyXKn8o77sE",
          "object": "model_permission",
          "created": 1695166152,
          "allow_create_engine": false,
          "allow_sampling": true,
          "allow_logprobs": true,
          "allow_search_indices": true,
          "allow_view": true,
          "allow_fine_tuning": false,
          "organization": "*",
          "group": null,
          "is_blocking": false
        }
      ],
      "root": "text-search-ada-query-001",
      "parent": null
    },
    {
      "id": "whisper-1",
      "object": "model",
      "created": 1677532384,
      "owned_by": "openai-internal",
      "permission": [
        {
          "id": "modelperm-MZ3upaPAtD0SZWUcGePyBpqI",
          "object": "model_permission",
          "created": 1697070197,
          "allow_create_engine": false,
          "allow_sampling": true,
          "allow_logprobs": true,
          "allow_search_indices": false,
          "allow_view": true,
          "allow_fine_tuning": false,
          "organization": "*",
          "group": null,
          "is_blocking": false
        }
      ],
      "root": "whisper-1",
      "parent": null
    },
    {
      "id": "code-search-ada-text-001",
      "object": "model",
      "created": 1651172507,
      "owned_by": "openai-dev",
      "permission": [
        {
          "id": "modelperm-MQOFciXDRywliXymhKJT5rUy",
          "object": "model_permission",
          "created": 1695166346,
          "allow_create_engine": false,
          "allow_sampling": true,
          "allow_logprobs": true,
          "allow_search_indices": true,
          "allow_view": true,
          "allow_fine_tuning": false,
          "organization": "*",
          "group": null,
          "is_blocking": false
        }
      ],
      "root": "code-search-ada-text-001",
      "parent": null
    },
    {
      "id": "babbage-code-search-text",
      "object": "model",
      "created": 1651172509,
      "owned_by": "openai-dev",
      "permission": [
        {
          "id": "modelperm-DE4WMTkago3EHJ0n1p2urLkw",
          "object": "model_permission",
          "created": 1695934039,
          "allow_create_engine": false,
          "allow_sampling": true,
          "allow_logprobs": true,
          "allow_search_indices": true,
          "allow_view": true,
          "allow_fine_tuning": false,
          "organization": "*",
          "group": null,
          "is_blocking": false
        }
      ],
      "root": "babbage-code-search-text",
      "parent": null
    },
    {
      "id": "code-search-babbage-code-001",
      "object": "model",
      "created": 1651172507,
      "owned_by": "openai-dev",
      "permission": [
        {
          "id": "modelperm-xuS8vuaSaa5jIQSmyP0S35Qd",
          "object": "model_permission",
          "created": 1695934060,
          "allow_create_engine": false,
          "allow_sampling": true,
          "allow_logprobs": true,
          "allow_search_indices": true,
          "allow_view": true,
          "allow_fine_tuning": false,
          "organization": "*",
          "group": null,
          "is_blocking": false
        }
      ],
      "root": "code-search-babbage-code-001",
      "parent": null
    },
    {
      "id": "ada-search-query",
      "object": "model",
      "created": 1651172505,
      "owned_by": "openai-dev",
      "permission": [
        {
          "id": "modelperm-kSLKiCrDbosTMvZnIt5wuEqE",
          "object": "model_permission",
          "created": 1695166172,
          "allow_create_engine": false,
          "allow_sampling": true,
          "allow_logprobs": true,
          "allow_search_indices": true,
          "allow_view": true,
          "allow_fine_tuning": false,
          "organization": "*",
          "group": null,
          "is_blocking": false
        }
      ],
      "root": "ada-search-query",
      "parent": null
    },
    {
      "id": "ada-code-search-text",
      "object": "model",
      "created": 1651172510,
      "owned_by": "openai-dev",
      "permission": [
        {
          "id": "modelperm-Ya8bDcPgbFbqFR0K44jdPN64",
          "object": "model_permission",
          "created": 1695166368,
          "allow_create_engine": false,
          "allow_sampling": true,
          "allow_logprobs": true,
          "allow_search_indices": true,
          "allow_view": true,
          "allow_fine_tuning": false,
          "organization": "*",
          "group": null,
          "is_blocking": false
        }
      ],
      "root": "ada-code-search-text",
      "parent": null
    },
    {
      "id": "text-search-curie-query-001",
      "object": "model",
      "created": 1651172509,
      "owned_by": "openai-dev",
      "permission": [
        {
          "id": "modelperm-7Y7YYZkPezf96wy8j5bphjjg",
          "object": "model_permission",
          "created": 1695149169,
          "allow_create_engine": false,
          "allow_sampling": true,
          "allow_logprobs": true,
          "allow_search_indices": true,
          "allow_view": true,
          "allow_fine_tuning": false,
          "organization": "*",
          "group": null,
          "is_blocking": false
        }
      ],
      "root": "text-search-curie-query-001",
      "parent": null
    },
    {
      "id": "text-davinci-002",
      "object": "model",
      "created": 1649880484,
      "owned_by": "openai",
      "permission": [
        {
          "id": "modelperm-eKMXF4rV1slbvrpgmDbTm3wD",
          "object": "model_permission",
          "created": 1695082227,
          "allow_create_engine": false,
          "allow_sampling": true,
          "allow_logprobs": true,
          "allow_search_indices": false,
          "allow_view": true,
          "allow_fine_tuning": false,
          "organization": "*",
          "group": null,
          "is_blocking": false
        }
      ],
      "root": "text-davinci-002",
      "parent": null
    },
    {
      "id": "text-embedding-ada-002",
      "object": "model",
      "created": 1671217299,
      "owned_by": "openai-internal",
      "permission": [
        {
          "id": "modelperm-iHRhxPwYPqj6AJE3PFzga9rD",
          "object": "model_permission",
          "created": 1695144609,
          "allow_create_engine": false,
          "allow_sampling": true,
          "allow_logprobs": true,
          "allow_search_indices": true,
          "allow_view": true,
          "allow_fine_tuning": false,
          "organization": "*",
          "group": null,
          "is_blocking": false
        }
      ],
      "root": "text-embedding-ada-002",
      "parent": null
    },
    {
      "id": "text-davinci-edit-001",
      "object": "model",
      "created": 1649809179,
      "owned_by": "openai",
      "permission": [
        {
          "id": "modelperm-bwEWUtGiBcdX0p1D1ayafH8w",
          "object": "model_permission",
          "created": 1690915020,
          "allow_create_engine": false,
          "allow_sampling": true,
          "allow_logprobs": true,
          "allow_search_indices": false,
          "allow_view": true,
          "allow_fine_tuning": false,
          "organization": "*",
          "group": null,
          "is_blocking": false
        }
      ],
      "root": "text-davinci-edit-001",
      "parent": null
    },
    {
      "id": "code-search-babbage-text-001",
      "object": "model",
      "created": 1651172507,
      "owned_by": "openai-dev",
      "permission": [
        {
          "id": "modelperm-JkzgCs0oPZaC1uCfO0n0ulYX",
          "object": "model_permission",
          "created": 1695934089,
          "allow_create_engine": false,
          "allow_sampling": true,
          "allow_logprobs": true,
          "allow_search_indices": true,
          "allow_view": true,
          "allow_fine_tuning": false,
          "organization": "*",
          "group": null,
          "is_blocking": false
        }
      ],
      "root": "code-search-babbage-text-001",
      "parent": null
    },
    {
      "id": "gpt-3.5-turbo-instruct-0914",
      "object": "model",
      "created": 1694122472,
      "owned_by": "system",
      "permission": [
        {
          "id": "modelperm-kMX0MSKsmmMHNQaGtK0ID5Nr",
          "object": "model_permission",
          "created": 1697244057,
          "allow_create_engine": false,
          "allow_sampling": true,
          "allow_logprobs": true,
          "allow_search_indices": false,
          "allow_view": true,
          "allow_fine_tuning": false,
          "organization": "*",
          "group": null,
          "is_blocking": false
        }
      ],
      "root": "gpt-3.5-turbo-instruct-0914",
      "parent": null
    },
    {
      "id": "ada",
      "object": "model",
      "created": 1649357491,
      "owned_by": "openai",
      "permission": [
        {
          "id": "modelperm-0JnkYlxDvw3kMS1mht3cmQKR",
          "object": "model_permission",
          "created": 1695934146,
          "allow_create_engine": false,
          "allow_sampling": true,
          "allow_logprobs": true,
          "allow_search_indices": false,
          "allow_view": true,
          "allow_fine_tuning": false,
          "organization": "*",
          "group": null,
          "is_blocking": false
        }
      ],
      "root": "ada",
      "parent": null
    },
    {
      "id": "text-ada-001",
      "object": "model",
      "created": 1649364042,
      "owned_by": "openai",
      "permission": [
        {
          "id": "modelperm-jRuB7xBCdj159SqaDmpPgeWO",
          "object": "model_permission",
          "created": 1690915029,
          "allow_create_engine": false,
          "allow_sampling": true,
          "allow_logprobs": true,
          "allow_search_indices": false,
          "allow_view": true,
          "allow_fine_tuning": false,
          "organization": "*",
          "group": null,
          "is_blocking": false
        }
      ],
      "root": "text-ada-001",
      "parent": null
    },
    {
      "id": "ada-similarity",
      "object": "model",
      "created": 1651172507,
      "owned_by": "openai-dev",
      "permission": [
        {
          "id": "modelperm-z4NZ5sCV2GsxBeNqzcNVhEvH",
          "object": "model_permission",
          "created": 1695166231,
          "allow_create_engine": false,
          "allow_sampling": true,
          "allow_logprobs": true,
          "allow_search_indices": true,
          "allow_view": true,
          "allow_fine_tuning": false,
          "organization": "*",
          "group": null,
          "is_blocking": false
        }
      ],
      "root": "ada-similarity",
      "parent": null
    },
    {
      "id": "code-search-ada-code-001",
      "object": "model",
      "created": 1651172507,
      "owned_by": "openai-dev",
      "permission": [
        {
          "id": "modelperm-9k7jVGlzTgB52nza3Po7ENUZ",
          "object": "model_permission",
          "created": 1695166414,
          "allow_create_engine": false,
          "allow_sampling": true,
          "allow_logprobs": true,
          "allow_search_indices": true,
          "allow_view": true,
          "allow_fine_tuning": false,
          "organization": "*",
          "group": null,
          "is_blocking": false
        }
      ],
      "root": "code-search-ada-code-001",
      "parent": null
    },
    {
      "id": "text-similarity-ada-001",
      "object": "model",
      "created": 1651172505,
      "owned_by": "openai-dev",
      "permission": [
        {
          "id": "modelperm-2l92GeLtllKOhDdWb6RzzfEN",
          "object": "model_permission",
          "created": 1695166256,
          "allow_create_engine": false,
          "allow_sampling": true,
          "allow_logprobs": true,
          "allow_search_indices": true,
          "allow_view": true,
          "allow_fine_tuning": false,
          "organization": "*",
          "group": null,
          "is_blocking": false
        }
      ],
      "root": "text-similarity-ada-001",
      "parent": null
    },
    {
      "id": "gpt-3.5-turbo-0301",
      "object": "model",
      "created": 1677649963,
      "owned_by": "openai",
      "permission": [
        {
          "id": "modelperm-I4IcSJFYZl2fIK0DPSBkgK3d",
          "object": "model_permission",
          "created": 1691712139,
          "allow_create_engine": false,
          "allow_sampling": true,
          "allow_logprobs": true,
          "allow_search_indices": false,
          "allow_view": true,
          "allow_fine_tuning": false,
          "organization": "*",
          "group": null,
          "is_blocking": false
        }
      ],
      "root": "gpt-3.5-turbo-0301",
      "parent": null
    },
    {
      "id": "gpt-3.5-turbo-instruct",
      "object": "model",
      "created": 1692901427,
      "owned_by": "system",
      "permission": [
        {
          "id": "modelperm-UK4BVstQKYTKaTj3zbCsKCGx",
          "object": "model_permission",
          "created": 1697244075,
          "allow_create_engine": false,
          "allow_sampling": true,
          "allow_logprobs": true,
          "allow_search_indices": false,
          "allow_view": true,
          "allow_fine_tuning": false,
          "organization": "*",
          "group": null,
          "is_blocking": false
        }
      ],
      "root": "gpt-3.5-turbo-instruct",
      "parent": null
    },
    {
      "id": "gpt-3.5-turbo-16k",
      "object": "model",
      "created": 1683758102,
      "owned_by": "openai-internal",
      "permission": [
        {
          "id": "modelperm-ws4f4bZHZRiHC2iI0emxIa4z",
          "object": "model_permission",
          "created": 1694726630,
          "allow_create_engine": false,
          "allow_sampling": true,
          "allow_logprobs": true,
          "allow_search_indices": false,
          "allow_view": true,
          "allow_fine_tuning": false,
          "organization": "*",
          "group": null,
          "is_blocking": false
        }
      ],
      "root": "gpt-3.5-turbo-16k",
      "parent": null
    },
    {
      "id": "text-search-curie-doc-001",
      "object": "model",
      "created": 1651172509,
      "owned_by": "openai-dev",
      "permission": [
        {
          "id": "modelperm-gMoCj5Vfel451hGbAp0kfKMH",
          "object": "model_permission",
          "created": 1695149177,
          "allow_create_engine": false,
          "allow_sampling": true,
          "allow_logprobs": true,
          "allow_search_indices": true,
          "allow_view": true,
          "allow_fine_tuning": false,
          "organization": "*",
          "group": null,
          "is_blocking": false
        }
      ],
      "root": "text-search-curie-doc-001",
      "parent": null
    },
    {
      "id": "gpt-4-0613",
      "object": "model",
      "created": 1686588896,
      "owned_by": "openai",
      "permission": [
        {
          "id": "modelperm-KaOvkLaF0m5Gvn1rZloQMZgq",
          "object": "model_permission",
          "created": 1697054229,
          "allow_create_engine": false,
          "allow_sampling": false,
          "allow_logprobs": false,
          "allow_search_indices": false,
          "allow_view": false,
          "allow_fine_tuning": false,
          "organization": "*",
          "group": null,
          "is_blocking": false
        }
      ],
      "root": "gpt-4-0613",
      "parent": null
    },
    {
      "id": "gpt-3.5-turbo-16k-0613",
      "object": "model",
      "created": 1685474247,
      "owned_by": "openai",
      "permission": [
        {
          "id": "modelperm-Hk9J34s4hlEGFpmvYhhEdeso",
          "object": "model_permission",
          "created": 1695243588,
          "allow_create_engine": false,
          "allow_sampling": true,
          "allow_logprobs": true,
          "allow_search_indices": false,
          "allow_view": true,
          "allow_fine_tuning": false,
          "organization": "*",
          "group": null,
          "is_blocking": false
        }
      ],
      "root": "gpt-3.5-turbo-16k-0613",
      "parent": null
    },
    {
      "id": "text-davinci-003",
      "object": "model",
      "created": 1669599635,
      "owned_by": "openai-internal",
      "permission": [
        {
          "id": "modelperm-OLYlW6I3e7VURS85ytehLxKj",
          "object": "model_permission",
          "created": 1696613334,
          "allow_create_engine": false,
          "allow_sampling": true,
          "allow_logprobs": true,
          "allow_search_indices": false,
          "allow_view": true,
          "allow_fine_tuning": false,
          "organization": "*",
          "group": null,
          "is_blocking": false
        }
      ],
      "root": "text-davinci-003",
      "parent": null
    },
    {
      "id": "gpt-4",
      "object": "model",
      "created": 1687882411,
      "owned_by": "openai",
      "permission": [
        {
          "id": "modelperm-rUjqpD58WNZrVBqYlZp7PLfX",
          "object": "model_permission",
          "created": 1697054232,
          "allow_create_engine": false,
          "allow_sampling": false,
          "allow_logprobs": false,
          "allow_search_indices": false,
          "allow_view": false,
          "allow_fine_tuning": false,
          "organization": "*",
          "group": null,
          "is_blocking": false
        }
      ],
      "root": "gpt-4",
      "parent": null
    },
    {
      "id": "text-curie-001",
      "object": "model",
      "created": 1649364043,
      "owned_by": "openai",
      "permission": [
        {
          "id": "modelperm-3D7myCVeMXS00PJdPESttrwj",
          "object": "model_permission",
          "created": 1695936121,
          "allow_create_engine": false,
          "allow_sampling": true,
          "allow_logprobs": true,
          "allow_search_indices": false,
          "allow_view": true,
          "allow_fine_tuning": false,
          "organization": "*",
          "group": null,
          "is_blocking": false
        }
      ],
      "root": "text-curie-001",
      "parent": null
    },
    {
      "id": "gpt-3.5-turbo",
      "object": "model",
      "created": 1677610602,
      "owned_by": "openai",
      "permission": [
        {
          "id": "modelperm-WVAcp6bvsxmZVeiS0E4BE5pf",
          "object": "model_permission",
          "created": 1696715453,
          "allow_create_engine": false,
          "allow_sampling": true,
          "allow_logprobs": true,
          "allow_search_indices": false,
          "allow_view": true,
          "allow_fine_tuning": false,
          "organization": "*",
          "group": null,
          "is_blocking": false
        }
      ],
      "root": "gpt-3.5-turbo",
      "parent": null
    },
    {
      "id": "curie",
      "object": "model",
      "created": 1649359874,
      "owned_by": "openai",
      "permission": [
        {
          "id": "modelperm-FwPnxb7Y7YHn9eMwTNgQyiJ4",
          "object": "model_permission",
          "created": 1695936177,
          "allow_create_engine": false,
          "allow_sampling": true,
          "allow_logprobs": true,
          "allow_search_indices": false,
          "allow_view": true,
          "allow_fine_tuning": false,
          "organization": "*",
          "group": null,
          "is_blocking": false
        }
      ],
      "root": "curie",
      "parent": null
    },
    {
      "id": "gpt-4-0314",
      "object": "model",
      "created": 1687882410,
      "owned_by": "openai",
      "permission": [
        {
          "id": "modelperm-uAZYNOQTI3UZMxYcK6iftTii",
          "object": "model_permission",
          "created": 1696968564,
          "allow_create_engine": false,
          "allow_sampling": false,
          "allow_logprobs": false,
          "allow_search_indices": false,
          "allow_view": false,
          "allow_fine_tuning": false,
          "organization": "*",
          "group": null,
          "is_blocking": false
        }
      ],
      "root": "gpt-4-0314",
      "parent": null
    }
  ]
}



from abstract_gui.SIMPLEGUI.old import text_to_key
class AbstractNavigationManager:  # Replace with your actual class name
    def __init__(self,selfs,window_mgr)->None:
        self.window_mgr = window_mgr
        self.selfs =selfs
        self.action_keys = ['prompt_data','request','query','chunk','instructions']
    def chunk_event_check(self)->None:
        """
        Checks if any events related to the chunk UI elements were triggered and performs the necessary actions.
        """
        self.event,self.values,self.window=args
        # Simplified method to determine the type and direction of navigation
        navigation_type, navigation_direction = self.parse_navigation_event(self.event)

        # Early exit if the event is not a navigation event
        if not navigation_type:
            return

        # Retrieve navigation data based on the event
        nav_data = self.get_navigation_data(navigation_type)

        # Update the section and subsection numbers based on navigation
        self.update_navigation_counters(nav_data, navigation_direction)

        # Update the display based on the updated navigation data
        self.update_display(nav_data)

    def parse_navigation_event(self, event:str)->((None,None) or (str,str)):
        """
        Parses the event to extract navigation type and direction.
        """
        if event.startswith('-') and event.endswith('-'):
            parts = event[1:-1].lower().split('_')
            if 'back' in parts or 'forward' in parts:
                nav_type = '_'.join(parts[:-2]) if 'section' in parts else parts[0]
                nav_direction = parts[-1]
                return nav_type, nav_direction
        return None, None

    def get_navigation_data(self, navigation_type:str)->dict:
        """
        Retrieves the necessary data for navigation based on the navigation type.
        """
        nav_data = {
            'data_type': navigation_type,
            'section_number': int(self.window_mgr.get_from_value(f"-{navigation_type.upper()}_SECTION_NUMBER-")),
            'number': self.get_sub_section_number(navigation_type),
            'reference_object': self.get_reference_object(navigation_type)
        }
        return nav_data

    def get_sub_section_number(self, navigation_type:str)->int:
        """
        Retrieves the current sub-section number based on navigation type.
        """
        spl = self.window_mgr.get_event().split('_')
        section = spl[1]
        self.number_key = f"-{navigation_type.upper()}{'_SECTION' if len(spl)>= 3 else ''}_NUMBER-"
        return int(self.window_mgr.get_from_value(self.number_key)) if self.window_mgr.exists(self.number_key) else None

    def get_reference_object(self, navigation_type:str)->list:
        """
        Retrieves the reference object based on navigation type.
        """
        reference_js = {
            "request": self.selfs.request_data_list,
            "prompt_data": self.selfs.prompt_data_list,
            "chunk": self.selfs.prompt_mgr.chunk_token_distributions,
            "query": self.selfs.prompt_mgr.chunk_token_distributions,
            "instructions":self.selfs.prompt_data_list
            
        }
        return reference_js.get(navigation_type, [])

    def update_navigation_counters(self, nav_data:dict, direction:str)->None:
        """
        Updates section and subsection numbers based on the navigation direction.
        """
        def get_adjusted_number(current_number:int, reference_obj:list)->int:
            return max(0, min(current_number, max(0, len(reference_obj))))
        reference_obj = nav_data['reference_object']
        current_section_number = self.selfs.display_number_tracker[nav_data['data_type']]
        current_section_number = get_adjusted_number(current_section_number, reference_obj)
        if 'SECTION' in self.number_key:
            max_value = max(0, len(reference_obj) - 1)
            increment = 1 if direction == 'forward' else -1
            nav_data['section_number'] = max(0, min(current_section_number + increment, max_value))
            nav_data["number"]=0
        elif nav_data['data_type'] in ["chunk", "query"]:
            reference_obj = reference_obj[current_section_number]
            current_chunk_number = self.selfs.display_number_tracker['chunk_number']
            current_chunk_number = get_adjusted_number(current_chunk_number, reference_obj)

            max_value = max(0, len(reference_obj) - 1)
            increment = 1 if direction == 'forward' else -1
            nav_data['number'] = max(0, min(current_chunk_number + increment, max_value))

        self.update_display(nav_data)

    
    def update_number_tracker(self,nav_data:dict)->None:
        for key in self.action_keys:
            #print(f"key: {key} in self.action_keys: {self.action_keys}")
            self.selfs.display_number_tracker[key]=nav_data["section_number"]
        self.selfs.display_number_tracker['chunk_number']=nav_data["number"]
        
    def update_data_displays(self,nav_data:dict)->None:
        self.selfs.update_request_data_display(nav_data["section_number"])
        self.selfs.update_prompt_data_display(nav_data["section_number"])
        action_keys = ['chunk','query']
        number_types = ['section_number','number']
        for action_key in self.action_keys:
            #print(f"action_key: {action_key} in self.action_keys: {self.action_keys}\n\n")
            for number_type in number_types:
                #print(f"number_type: {number_type} in number_types: {number_types}\n\n")
                #print(f"updating nav_data[number_type]: {nav_data[number_type]}\n\n")
                self.window_mgr.update_value(text_to_key(f'{action_key}_{number_type}'),nav_data[number_type])
    def update_display(self, nav_data:dict)->None:
        """
        Updates the display based on the navigation data.
        """
        #print(f"nav_data == {nav_data}\n\n")
        self.update_number_tracker(nav_data)
        
        self.update_data_displays(nav_data)
        section_number = nav_data["section_number"]
        #print(f"section_number: {section_number}\n\n")
        #print(f"self.selfs.prompt_mgr.chunk_token_distributions == {self.selfs.prompt_mgr.chunk_token_distributions}\n\n")
        chunk_section = max(0, min(section_number, len(self.selfs.prompt_mgr.chunk_token_distributions)-1))
        chunk_token_distributions = self.selfs.prompt_mgr.chunk_token_distributions[chunk_section]
        #print(f"chunk_token_distributions: {chunk_token_distributions}\n\n")
        base_chunk_token_distribution = chunk_token_distributions[0]
        #print(f"base_chunk_token_distribution: {base_chunk_token_distribution}\n\n")
        base_chunk_data_token_distribution = base_chunk_token_distribution['chunk']['data']
        #print(f"base_chunk_data_token_distribution: {base_chunk_data_token_distribution}\n\n")
        self.window_mgr.update_value("-CHUNK_SECTIONED_DATA-",base_chunk_data_token_distribution)
        chunk_section = max(0, min(nav_data['section_number'], len(self.selfs.prompt_mgr.chunk_token_distributions)-1))
        self.window_mgr.update_value(key='-QUERY-',value=self.selfs.prompt_mgr.create_prompt(chunk_token_distribution_number=chunk_section,chunk_number=0))
        self.selfs.update_chunk_info(nav_data['section_number'],nav_data["number"])
        # Update the display logic here based on nav_data
        self.selfs.update_bool_instructions()




from abstract_utilities import *
def get_whole_brackets(obj,start=None):
    chars=''
    start = start or 0
    brackets_js = {'(':start,')':0}
    for char in str(obj):
        for key,value in brackets_js.items():
            if char and not char.replace(key,''):
                brackets_js[key]+=1
                vals = list(brackets_js.values())
                if vals[0] == vals[1]:
                    chars+=char
                    return chars
                break
        chars+=char
    return chars     
files = """/home/computron/Documents/pythonTools/modules/abstract_ai/src/abstract_ai/gpt_classes/nogui_selection/abstract_database_cmd.py:    response = input(f"{text} {choices}:({default}) ") or default
/home/computron/Documents/pythonTools/modules/abstract_ai/src/abstract_ai/gpt_classes/nogui_selection/abstract_database_cmd.py:    return input(f"Enter {typ} value {action} in {parentObject}: ")
/home/computron/Documents/pythonTools/modules/abstract_ai/src/abstract_ai/gpt_classes/nogui_selection/abstract_database_cmd.py:                input(rows)
/home/computron/Documents/pythonTools/modules/abstract_ai/src/abstract_ai/gpt_classes/nogui_selection/abstract_database_cmd.py:            value = input(f"Enter value to filter by {column_name} (or press Enter to skip): ")
/home/computron/Documents/pythonTools/modules/abstract_ai/src/abstract_ai/gpt_classes/nogui_selection/abstract_database_cmd.py:            choice = input("Enter your choice: ")
/home/computron/Documents/pythonTools/modules/abstract_ai/src/abstract_ai/gpt_classes/nogui_selection/abstract_database_cmd.py:                file_path = input("Enter the file path to save the Excel file (default: output.xlsx): ") or "output.xlsx"
/home/computron/Documents/pythonTools/modules/abstract_ai/src/abstract_ai/gpt_classes/nogui_selection/abstract_database_cmd.py:                error_message = input("Enter the error message to search for (e.g., 'Method not found'): ")
/home/computron/Documents/pythonTools/modules/abstract_ai/src/abstract_ai/gpt_classes/nogui_selection/abstract_database_cmd.py:                confirm = input(f"Are you sure you want to delete the entire table '{table_name}'? (y/n): ").lower()
/home/computron/Documents/pythonTools/modules/abstract_ai/src/abstract_ai/gpt_classes/nogui_selection/db_query.py:    input()
/home/computron/Documents/pythonTools/modules/abstract_ai/src/abstract_ai/gpt_classes/file_section/responseContentParser.py:        input(description_data[title])
/home/computron/Documents/pythonTools/modules/abstract_ai/src/abstract_ai/gpt_classes/file_section/JsonHandler.py:    input(s)
/home/computron/Documents/pythonTools/modules/abstract_ai/src/abstract_ai/gpt_classes/file_section/JsonHandler.py:        input(output_string)
/home/computron/Documents/pythonTools/modules/abstract_ai/src/abstract_ai/specializations/responseContentParser.py:            input('out of o order')
/home/computron/Documents/pythonTools/modules/abstract_ai/src/abstract_ai/specializations/JsonHandler.py:    input(s)""".split('/home/')
for each in [each for each in files if each]:
    file_spl = each.split('.py:')
    file_path = f"/home/{file_spl[0]}.py"
    data = read_from_file(file_path)
    input_data = file_spl[-1]
    clean_input_data = eatAll(input_data,[' ','\t','\n'])
    if input_data in data:
        for chunk in data.split(input_data)[1:]:
            if chunk in data and 'input(f"Enter ' not in input_data and 'input("Enter ' not in input_data:
                input_data = f"{input_data}{chunk}"
                
                whole_bracket = get_whole_brackets(input_data)
                response = input(f"delete {whole_bracket}")
                if response == 'y':
                    data = data.replace(whole_bracket,'')
                    write_to_file(contents=data,file_path=file_path)
                    data = read_from_file(file_path)



from .prompt_tabs import prompt_tabs
from .progress_frame import get_progress_frame,get_output_options
from .gui_utilities import utilities
from .gui_utils import *
def get_total_layout()->list:
    return [
        [get_progress_frame()],
        [get_output_options()],
        [prompt_tabs(),get_column([utilities()])]
        ]

